; Example Program ADC01: Analog-to-Digital Conversion
; Chin-Shiuh Shieh
; 2003-03-20
;
; Pin Assignment
CLCD_RS		EQU	P2.0	; 0 for Instruction, 1 for Data
CLCD_RW		EQU	P2.1	; 0 for Write, 1 for Read
CLCD_E		EQU	P2.2	; 1 for Read, 1->0 for Write
CLCD_Port	EQU	P1
; Memory Map
ADC_ADRS	EQU	0000h	; ADC Address
; Memory Allocation
ADC_BUSY	EQU	00h		; Bit for ADC Status
;
PROG	EQU		0000h
		ORG		PROG+0000h
		JMP		START
;Interrupt Vector Table
		ORG		PROG+0003h
		LCALL	INT0_ISR
		RETI
		ORG		PROG+000Bh
		LCALL	T0_ISR
		RETI
		ORG		PROG+0013h
		LCALL	INT1_ISR
		RETI
		ORG		PROG+001Bh
		LCALL	T1_ISR
		RETI
		ORG		PROG+0023h
		LCALL	UART_ISR
		RETI
;
		ORG		PROG+0030h
START:
		SETB	IT0			; Set External Interrupt 0 to be falling edge triggered
		SETB	EX0			; Enable External Interrut 0
		SETB	EA			; Enable Interrupt
		CALL	CLCD_INIT
		MOV		A,#80h		; Move cursor to Row 0, Column 0
		CALL	CLCD_WI
		MOV		DPTR,#M_WEL
		CALL	CLCD_PUTS
		MOV		A,#0C0h		; Move cursor to Row 1, Column 0
		CALL	CLCD_WI
		MOV		DPTR,#M_ADC
		CALL	CLCD_PUTS
;
		CLR		ADC_BUSY
LOOP:	JB		ADC_BUSY,LOOP
		SETB	ADC_BUSY
		MOV		DPTR,#ADC_ADRS
		MOVX	@DPTR,A		; Start ADC
		CALL	DELAY
		SJMP	LOOP
;
CLCD_WAIT:
		PUSH	ACC
        MOV     CLCD_Port,#0FFh
		CLR		CLCD_RS
		SETB	CLCD_RW
CLCD_BUSY:
		SETB	CLCD_E
        MOV     A,CLCD_Port
		CLR		CLCD_E
		JB		ACC.7,CLCD_BUSY
		POP		ACC
		RET
;
CLCD_WI:
		CALL	CLCD_WAIT
		CLR		CLCD_RS
		CLR		CLCD_RW
        MOV     CLCD_Port,A
		SETB	CLCD_E
		CLR		CLCD_E
		RET
;
CLCD_WD:
		CALL	CLCD_WAIT
		SETB	CLCD_RS
		CLR		CLCD_RW
        MOV     CLCD_Port,A
		SETB	CLCD_E
		CLR		CLCD_E
		RET
;
CLCD_INIT:
        MOV     A,#00111000b	; 8-bit interface, 2 lines, 5x7 dots
        CALL    CLCD_WI
        MOV     A,#00001111b	; display on, cursor on, cursor blinking
        CALL    CLCD_WI
        MOV     A,#00000110b	; move cursor right when read/write
        CALL    CLCD_WI
        MOV     A,#00000001b	; clear display, cursor home
        CALL    CLCD_WI
        RET
;
CLCD_PUTS:
        MOV     A,#00h
        MOVC    A,@A+DPTR
        CJNE    A,#00h,CLCD_PUTS_LOOP
        RET
CLCD_PUTS_LOOP:
        CALL    CLCD_WD
        INC     DPTR
        SJMP    CLCD_PUTS
;
M_WEL:  DB      "Hello, 8051!",00h
M_ADC:	DB      "ADC: ",00h
;
CLCD_SHOW_BYTE:
		MOV		B,A
        SWAP    A
        ANL     A,#0Fh
        MOV     DPTR,#ASCII
        MOVC    A,@A+DPTR
        CALL	CLCD_WD
        MOV		A,B
        ANL     A,#0Fh
        MOV     DPTR,#ASCII
        MOVC    A,@A+DPTR
        CALL	CLCD_WD
		RET
;
ASCII:	DB		"0123456789ABCDEF"
;
DELAY:	MOV		R0,#00h
DELAY1:	MOV		R1,#00h
DELAY2:	DJNZ	R1,DELAY2
		DJNZ	R0,DELAY1
		RET
;
INT0_ISR:
		MOV		A,#0C5h		; Move cursor to Row 1, Column 5
		CALL	CLCD_WI
		MOV		DPTR,#ADC_ADRS
		MOVX	A,@DPTR
		CALL	CLCD_SHOW_BYTE
		CLR		ADC_BUSY
		RET
T0_ISR:
		RET
INT1_ISR:
		RET
T1_ISR:
		RET
UART_ISR:
		RET
;
        END
